@@ -1,16 +1,16 @@
 /*****************************************************************************
- * Copyright (c) 2011 Sven Brauch <svenbrauch@googlemail.com>                *
+ * Copyright (c) 2011-2013 Sven Brauch <svenbrauch@googlemail.com>           *
  *                                                                           *
  * This program is free software; you can redistribute it and/or             *
  * modify it under the terms of the GNU General Public License as            *
  * published by the Free Software Foundation; either version 2 of            *
  * the License, or (at your option) any later version.                       *
- *                                                                           *           
+ *                                                                           *
  * This program is distributed in the hope that it will be useful,           *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
  * GNU General Public License for more details.                              *
- *                                                                           *   
+ *                                                                           *
  * You should have received a copy of the GNU General Public License         *
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
  *****************************************************************************
@@ -21,7 +21,6 @@
 
 #include <language/codecompletion/codecompletioncontext.h>
 #include <language/duchain/duchainpointer.h>
-#include <kdev-pg-memory-pool.h>
 #include <QStack>
 
 #include <QObject>
@@ -73,7 +72,8 @@ public:
         FunctionCallCompletion,
         InheritanceCompletion,
         RaiseExceptionCompletion,
-        GeneratorVariableCompletion
+        GeneratorVariableCompletion,
+        StringFormattingCompletion
     };
     
     enum ItemTypeHint {
@@ -81,13 +81,20 @@ public:
         IterableRequested
     };
     
-    PythonCodeCompletionContext(DUContextPointer context, const QString& text, const KDevelop::CursorInRevision& position,
+    PythonCodeCompletionContext(DUContextPointer context, const QString& text, const QString& followingText,
+                                const KDevelop::CursorInRevision& position,
                                 int depth, const PythonCodeCompletionWorker* parent);
+
+    CompletionContextType completionContextType();
+
     ItemTypeHint itemTypeHint();
+
     /**
      * @brief The "interface method" which returns all the completion items to kdevelop.
      **/
     virtual QList< KDevelop::CompletionTreeItemPointer > completionItems(bool& abort, bool fullCompletion = true);
+
+    virtual QList< CompletionTreeElementPointer > ungroupedElements();
     
     /**
      * @brief Get all possible items matching the specified dot-seperated search string.
@@ -129,18 +136,38 @@ private:
     PythonCodeCompletionContext(DUContextPointer context, const QString& remainingText,
                                 QString calledFunction, int depth, int alreadyGivenParameters, CodeCompletionContext* child);
     void summonParentForEventualCall(TokenList tokens, const QString& text);
+    /// Get possible "add import..." items for an expression.
+    QList< CompletionTreeItemPointer > getMissingIncludeItems(QString forString);
+    void eventuallyAddGroup(QString name, int priority, QList<CompletionTreeItemPointer> items);
+
+private:
+    /// Item generating functions
+    using ItemList = QList<CompletionTreeItemPointer>;
+    ItemList shebangItems();
+    ItemList generatorItems();
+    ItemList functionCallItems();
+    ItemList defineItems();
+    ItemList raiseItems();
+    ItemList importFileItems();
+    ItemList inheritanceItems();
+    ItemList memberAccessItems();
+    ItemList stringFormattingItems();
+    ItemList keywordItems();
+    ItemList classMemberInitItems();
+
+private:
     CompletionContextType m_operation;
     ItemTypeHint m_itemTypeHint;
     QStack<ProjectFolderItem*> m_folderStack;
     int m_maxFolderScanDepth;
     QStringList m_searchingForModule;
     QString m_searchImportItemsInModule;
-    const PythonCodeCompletionWorker* worker;
     KUrl m_workingOnDocument;
     
     CodeCompletionContext* m_child;
     
     QString m_guessTypeOfExpression;
+    QString m_followingText;
     
     QString m_indent;
     KDevelop::CursorInRevision m_position;
@@ -149,6 +176,10 @@ private:
     int m_alreadyGivenParametersCount;
 
     QString m_matchAgainst;
+
+    bool m_fullCompletion;
+
+    QList<CompletionTreeElementPointer> m_storedGroups;
 };
 
 }
