@@ -19,7 +19,6 @@
 #include "functiondeclaration.h"
 
 #include <language/codecompletion/normaldeclarationcompletionitem.h>
-#include <language/duchain/functiondeclaration.h>
 #include <language/codecompletion/codecompletionmodel.h>
 #include <language/duchain/types/functiontype.h>
 #include <language/duchain/aliasdeclaration.h>
@@ -29,9 +28,11 @@
 #include <KLocalizedString>
 
 #include "duchain/navigation/navigationwidget.h"
-#include <types/variablelengthcontainer.h>
+#include "types/variablelengthcontainer.h"
 #include "codecompletion/helpers.h"
 #include "declaration.h"
+#include "declarations/functiondeclaration.h"
+#include "duchain/helpers.h"
 
 
 using namespace KDevelop;
@@ -131,35 +132,33 @@ void FunctionDeclarationCompletionItem::setIsImportItem(bool isImportItem)
 void FunctionDeclarationCompletionItem::executed(KTextEditor::Document* document, const KTextEditor::Range& word)
 {
     kDebug() << "FunctionDeclarationCompletionItem executed";
-    DeclarationPointer decl(declaration());
-    DUChainPointer<FunctionDeclaration> fdecl;
-    AliasDeclaration* alias = dynamic_cast<AliasDeclaration*>(decl.data());
-    if ( alias ) {
-        DUChainReadLocker lock(DUChain::lock());
-        fdecl = dynamic_cast<FunctionDeclaration*>(alias->aliasedDeclaration().declaration());
-    }
-    else {
-        fdecl = decl.dynamicCast<FunctionDeclaration>();
-    }
-    if ( ! fdecl.data() || ! fdecl.dynamicCast<FunctionDeclaration>() ) {
+    DeclarationPointer resolvedDecl(Helper::resolveAliasDeclaration(declaration().data()));
+    DUChainReadLocker lock;
+    QPair<FunctionDeclarationPointer, bool> fdecl = Helper::functionDeclarationForCalledDeclaration(resolvedDecl);
+    lock.unlock();
+    if ( ! fdecl.first && (! resolvedDecl || ! resolvedDecl->abstractType()
+                           || resolvedDecl->abstractType()->whichType() != AbstractType::TypeStructure) ) {
         kError() << "ERROR: could not get declaration data, not executing completion item!";
         return;
     }
-    kDebug() << "declaration data: " << fdecl.data();
     QString suffix = "()";
-    KTextEditor::Range checkSuffix(word.end().line(), word.end().column(), word.end().line(), word.end().column() + 2);
-    if ( m_isImportItem || document->text(checkSuffix) == "()" ) {
+    KTextEditor::Range checkPrefix(word.start().line(), 0, word.start().line(), word.start().column());
+    KTextEditor::Range checkSuffix(word.end().line(), word.end().column(), word.end().line(), document->lineLength(word.end().line()));
+    if ( m_isImportItem || document->text(checkSuffix).trimmed().startsWith('(')
+         || document->text(checkPrefix).trimmed().endsWith('@')
+         || (fdecl.first && Helper::findDecoratorByName(fdecl.first.data(), QLatin1String("property"))) )
+    {
         // don't insert brackets if they're already there,
-        // or if the item is an import item.
+        // the item is a decorator, or if it's an import item.
         suffix = "";
     }
     // place cursor behind bracktes by default
     int skip = 2;
-    if ( fdecl.data()->type<FunctionType>()->arguments().length() != 0 ) {
+    if ( fdecl.first && fdecl.first->type<FunctionType>() && fdecl.first->type<FunctionType>()->arguments().length() != 0 ) {
         // place cursor in brackets if there's parameters
         skip = 1;
     }
-    document->replaceText(word, decl.data()->identifier().toString() + suffix);
+    document->replaceText(word, declaration()->identifier().toString() + suffix);
     if ( View* view = document->activeView() ) {
         view->setCursorPosition( Cursor(word.end().line(), word.end().column() + skip) );
     }
@@ -167,4 +166,4 @@ void FunctionDeclarationCompletionItem::executed(KTextEditor::Document* document
 
 FunctionDeclarationCompletionItem::~FunctionDeclarationCompletionItem() { }
 
-}
\ No newline at end of file
+}
