@@ -30,6 +30,8 @@
 #include <QtTest/QTest>
 #include <KUrl>
 #include <KStandardDirs>
+#include <KTextEditor/Editor>
+#include <KService>
 
 #include "codecompletion/context.h"
 #include "codecompletion/helpers.h"
@@ -38,6 +40,9 @@ using namespace KDevelop;
 
 QTEST_MAIN(Python::PyCompletionTest)
 
+Q_DECLARE_METATYPE(QList<Python::RangeInString>)
+Q_DECLARE_METATYPE(KTextEditor::Range)
+
 static int testId = 0;
 static QString basepath = "/tmp/__kdevpythoncompletiontest.dir/";
 static QFSFileEngine fileEngine;
@@ -181,6 +186,14 @@ void PyCompletionTest::testExpressionParser_data()
 
 const QList<CompletionTreeItem*> PyCompletionTest::invokeCompletionOn(const QString& initCode, const QString& invokeCode)
 {
+    CompletionParameters data = prepareCompletion(initCode, invokeCode);
+    return runCompletion(data);
+}
+
+const CompletionParameters PyCompletionTest::prepareCompletion(const QString& initCode, const QString& invokeCode)
+{
+    CompletionParameters completion_data;
+
     QString filename = nextFilename();
     QFile fileptr(filename);
     fileptr.open(QIODevice::WriteOnly);
@@ -199,22 +212,29 @@ const QList<CompletionTreeItem*> PyCompletionTest::invokeCompletionOn(const QStr
     QString allCode = copy.replace("%INVOKE", invokeCode);
     
     QStringList lines = allCode.split('\n');
-    CursorInRevision cursorAt = CursorInRevision::invalid();
+    completion_data.cursorAt = CursorInRevision::invalid();
     for ( int i = 0; i < lines.length(); i++ ) {
-        if ( int j = lines.at(i).indexOf("%CURSOR") != -1 ) {
-            cursorAt = CursorInRevision(i, j);
+        int j = lines.at(i).indexOf("%CURSOR");
+        if ( j != -1 ) {
+            completion_data.cursorAt = CursorInRevision(i, j);
             break;
         }
     }
-    Q_ASSERT(cursorAt.isValid());
+    Q_ASSERT(completion_data.cursorAt.isValid());
     // codeCompletionContext only gets passed the text until the place where completion is invoked
-    QString snip = allCode.mid(0, allCode.indexOf("%CURSOR"));
+    completion_data.snip = allCode.mid(0, allCode.indexOf("%CURSOR"));
+    completion_data.remaining = allCode.mid(allCode.indexOf("%CURSOR") + 7);
     
     DUChainReadLocker lock;
-    DUContextPointer contextAtCursor = DUContextPointer(topContext->findContextAt(cursorAt, true));
-    Q_ASSERT(contextAtCursor);
+    completion_data.contextAtCursor = DUContextPointer(topContext->findContextAt(completion_data.cursorAt, true));
+    Q_ASSERT(completion_data.contextAtCursor);
     
-    PythonCodeCompletionContext* context = new PythonCodeCompletionContext(contextAtCursor, snip, cursorAt, 0, 0);
+    return completion_data;
+}
+
+const QList<CompletionTreeItem*> PyCompletionTest::runCompletion(const CompletionParameters parameters)
+{
+    PythonCodeCompletionContext* context = new PythonCodeCompletionContext(parameters.contextAtCursor, parameters.snip, parameters.remaining, parameters.cursorAt, 0, 0);
     bool abort = false;
     QList<CompletionTreeItem*> items;
     foreach ( CompletionTreeItemPointer ptr, context->completionItems(abort, true) ) {
@@ -270,7 +290,7 @@ void PyCompletionTest::testImportCompletion()
     QFETCH(QString, invokeCode);
     QFETCH(QString, completionCode);
     QFETCH(QString, expectedItem);
-    
+
     if ( expectedItem == "EMPTY" ) {
         QVERIFY(completionListIsEmpty(invokeCode, completionCode));
     }
@@ -429,6 +449,29 @@ void PyCompletionTest::testImplementMethodCompletion_data()
                                      " def some_method2(param):\n  pass\n  pass\n %INVOKE" << "\n \n \n def %CURSOR";
 }
 
+void PyCompletionTest::testAutoBrackets()
+{
+    QList< CompletionTreeItem* > items = invokeCompletionOn("class Foo:\n @property\n def myprop(self): pass\n"
+                                                            "a=Foo()\n%INVOKE", "a.%CURSOR");
+    QVERIFY(containsItemForDeclarationNamed(items, "myprop"));
+    CompletionTreeItem* item = 0;
+    foreach ( CompletionTreeItem* ptr, items ) {
+        if ( ptr->declaration() ) {
+            if ( ptr->declaration()->identifier().toString() == "myprop" ) {
+                item = ptr;
+                break;
+            }
+        }
+    }
+    QVERIFY(item);
+    KService::Ptr documentService = KService::serviceByDesktopPath("katepart.desktop");
+    QVERIFY(documentService);
+    KTextEditor::Document* document = documentService->createInstance<KTextEditor::Document>(this);
+    QVERIFY(document);
+    item->execute(document, KTextEditor::Range(0, 0, 0, 0));
+    QCOMPARE(document->text(), QLatin1String("myprop"));
+}
+
 void PyCompletionTest::testExceptionCompletion()
 {
     QList< CompletionTreeItem* > items = invokeCompletionOn("localvar = 3\nraise %INVOKE", "%CURSOR");
@@ -457,6 +500,188 @@ void PyCompletionTest::testInheritanceCompletion()
     QVERIFY(containsItemForDeclarationNamed(items, "blubb"));
 }
 
+void PyCompletionTest::testAddImportCompletion()
+{
+    QFETCH(QString, completionCode);
+    QFETCH(QString, invokeCode);
+    QFETCH(int, expectedItems);
+
+    QCOMPARE(invokeCompletionOn(completionCode, invokeCode).size(), expectedItems);
 }
 
+void PyCompletionTest::testAddImportCompletion_data()
+{
+    QTest::addColumn<QString>("completionCode");
+    QTest::addColumn<QString>("invokeCode");
+    QTest::addColumn<int>("expectedItems");
 
+    QTest::newRow("has_entry_when_necessary") << "toplevelmodule%INVOKE" << ".%CURSOR" << 1;
+    QTest::newRow("has_no_when_not_necessary") << "toplevelmodule = 3;\ntoplevelmodule%INVOKE" << ".%CURSOR" << 0;
+}
+
+void PyCompletionTest::testFunctionDeclarationCompletion()
+{
+    QFETCH(QString, completionCode);
+    QFETCH(QString, invokeCode);
+    QFETCH(KTextEditor::Range, executeRange);
+    QFETCH(QString, expectedReplacement);
+
+    QString documentCode = completionCode;
+    documentCode.replace("%INVOKE", invokeCode).replace("%CURSOR", "");
+
+    QString expectedCode = completionCode;
+    expectedCode.replace("%INVOKE", expectedReplacement);
+
+    const QList<CompletionTreeItem *> completionItems = invokeCompletionOn(completionCode, invokeCode);
+
+    QVERIFY( ! completionItems.isEmpty() );
+
+    KService::Ptr documentService = KService::serviceByDesktopPath("katepart.desktop");
+    QVERIFY(documentService);
+    KTextEditor::Document* document = documentService->createInstance<KTextEditor::Document>(this);
+    QVERIFY(document);
+    document->setText(documentCode);
+
+    completionItems.first()->execute(document, executeRange);
+    QCOMPARE(document->text(), expectedCode);
+}
+
+void PyCompletionTest::testFunctionDeclarationCompletion_data()
+{
+    QTest::addColumn<QString>("completionCode");
+    QTest::addColumn<QString>("invokeCode");
+    QTest::addColumn<KTextEditor::Range>("executeRange");
+    QTest::addColumn<QString>("expectedReplacement");
+
+    QTest::newRow("func_decl_no_parens") << "def foo():\n  pass\n%INVOKE" << "foo%CURSOR" << KTextEditor::Range(2, 0, 2, 3)
+                                         << "foo()";
+
+    QTest::newRow("func_decl_existing_parens") << "def foo():\n  return 0\nbar = %INVOKE" << "foo%CURSOR()" << KTextEditor::Range(2, 6, 2, 9)
+                                           << "foo()";
+
+    QTest::newRow("decorator_no_parens") << "def mydecorator():\n  pass\nclass Foo:\n  %INVOKE\n  def bar():\n    pass" << "@mydecorator%CURSOR" << KTextEditor::Range(3, 3, 3, 15)
+                                         << "@mydecorator";
+
+    QTest::newRow("class_name_no_constructor_parens") << "class Foo:\n  pass\nbar = %INVOKE" << "Foo%CURSOR" << KTextEditor::Range(2, 6, 2, 9)
+                                                      << "Foo()";
+
+    QTest::newRow("class_name_explicit_constructor_parens") << "class Foo:\n  def __init__(self):\n    pass\nbar = %INVOKE" << "Foo%CURSOR"
+                                                        << KTextEditor::Range(3, 6, 3, 9)
+                                                        << "Foo()";
+}
+
+void PyCompletionTest::testStringFormattingCompletion()
+{
+    QFETCH(QString, completionCode);
+    QFETCH(QString, invokeCode);
+    QFETCH(QString, expectedItem);
+    QFETCH(bool, expectedPresent);
+
+    QCOMPARE(itemInCompletionList(completionCode, invokeCode, expectedItem), expectedPresent);
+}
+
+void PyCompletionTest::testStringFormattingCompletion_data()
+{
+    QTest::addColumn<QString>("completionCode");
+    QTest::addColumn<QString>("invokeCode");
+    QTest::addColumn<QString>("expectedItem");
+    QTest::addColumn<bool>("expectedPresent");
+
+    QTest::newRow("sq_string") << "'foo %INVOKE'" << "%CURSOR" << "{0}" << true;
+    QTest::newRow("dq_string") << "\"foo %INVOKE bar\"" << "%CURSOR" << "{0}" << true;
+    QTest::newRow("sq_ml_string") << "'''foo\n\n%INVOKE\nbar'''" << "%CURSOR" << "{0}" << true;
+    QTest::newRow("dq_ml_string") << "\"\"\"foo\nbar %INVOKE baz\"\"\"" << "%CURSOR" << "{0}" << true;
+    QTest::newRow("auto_id") << "\"foo {0} bar {1} baz %INVOKE\"" << "%CURSOR" << "{2}" << true;
+
+    QTest::newRow("format_suggestions_conversion") << "\"foo {0} bar %INVOKE\"" << "{1}%CURSOR" << "{1!r}" << true;
+    QTest::newRow("format_suggestions_spec") << "\"foo {0} bar {1}%INVOKE baz\"" << "{2}%CURSOR" << "{2:%}" << true;
+
+    QTest::newRow("incompatible_suggestions_conversion") << "\"foo %INVOKE bar\"" << "{0:%}%CURSOR" << "{0!s:%}" << false;
+    QTest::newRow("incompatible_suggestions_spec") << "\"foo %INVOKE bar\"" << "{0!s}%CURSOR" << "{0!s:%}" << false;
+
+    QTest::newRow("alignment_for_strings") << "\"foo %INVOKE bar\"" << "{0!s}%CURSOR" << "{0!s:^${width}}" << true;
+    QTest::newRow("conversion_for_aliged_strings") << "\"foo %INVOKE bar\"" << "{0!s}%CURSOR" << "{0!s:^${width}}" << true;
+
+}
+
+void PyCompletionTest::testStringFormatter()
+{
+    QFETCH(QString, string);
+    QFETCH(int, expectedId);
+    QFETCH(QList<RangeInString>, expectedVariablePositions);
+
+    StringFormatter f(string);
+
+    int id = f.nextIdentifierId();
+    QCOMPARE(id, expectedId);
+
+    for (int i = 0; i < string.size(); i++) {
+        bool expectedInsideVariable = false;
+        foreach (RangeInString range, expectedVariablePositions) {
+            if (i >= range.beginIndex && i <= range.endIndex) {
+                expectedInsideVariable = true;
+                break;
+            }
+        }
+        QCOMPARE(f.isInsideReplacementVariable(i), expectedInsideVariable);
+    }
+}
+
+void PyCompletionTest::testStringFormatter_data()
+{
+    QTest::addColumn<QString>("string");
+    QTest::addColumn<int>("expectedId");
+    QTest::addColumn<QList<RangeInString> >("expectedVariablePositions");
+
+    QTest::newRow("sl_string") << "\"foo {0} bar {1}\"" << 2
+                               << (QList<RangeInString>() << RangeInString(5, 8) << RangeInString(13, 16));
+
+    QTest::newRow("ml_string") << "\"\"\"foo {0} \n\nbar {1} \n{foo} {2}\nbaz\"\"\"" << 3
+                               << (QList<RangeInString>() << RangeInString(7, 10) << RangeInString(17, 20)
+                                   << RangeInString(22, 27) << RangeInString(28, 31));
+
+    QTest::newRow("containing_quotes") << "'''foo {0}\nbar\n{1}{0} \\' \\\" \\\" {2} {foo} \n\\'\n {3}baz'''" << 4
+                                       << (QList<RangeInString>() << RangeInString(7, 10) << RangeInString(15, 18)
+                                           << RangeInString(18, 21) << RangeInString(31, 34) << RangeInString(35, 40)
+                                           << RangeInString(46, 49));
+}
+
+QString repeat_distinct(const QString& code, int count) {
+    QString result;
+    QString line;
+    for ( int i = 0; i < count; i++ ) {
+        line = code;
+        result.append(line.replace(QString("%X"), QString::number(i)));
+    }
+    return result;
+}
+
+void PyCompletionTest::completionBenchTest()
+{
+    QFETCH(QString, completionCode);
+    QFETCH(QString, invokeCode);
+
+    CompletionParameters data = prepareCompletion(completionCode, invokeCode);
+    QBENCHMARK {
+        runCompletion(data);
+    }
+}
+
+void PyCompletionTest::completionBenchTest_data()
+{
+    QTest::addColumn<QString>("completionCode");
+    QTest::addColumn<QString>("invokeCode");
+
+    QTest::newRow("variable_completion") << "a0 = 2\n%INVOKE" << "b = a%CURSOR";
+    QTest::newRow("no_items") << "%INVOKE" << "def func(%CURSOR";
+    QTest::newRow("function") << "%INVOKE" << "foo(%CURSOR";
+    QTest::newRow("deep_function") << "foo(bar(baz(bang([]%INVOKE))))" << ".%CURSOR";
+    QTest::newRow("class_completion") << "class my(): pass\nd = my()\n%INVOKE" << "d.%CURSOR";
+
+    QString many_globals = repeat_distinct("a%X=%X\n", 1000);
+
+    QTest::newRow("function_many_globals") << many_globals + "%INVOKE" << "foo(%CURSOR";
+    QTest::newRow("variable_completion_many_globals") << many_globals + "%INVOKE" << "b = a%CURSOR";
+}
+
+}
