@@ -30,6 +30,7 @@
 #include "expressionvisitor.h"
 #include "pythoneditorintegrator.h"
 #include "helpers.h"
+#include "assistants/missingincludeassistant.h"
 
 #include <language/duchain/functiondeclaration.h>
 #include <language/duchain/declaration.h>
@@ -80,6 +81,8 @@ void DeclarationBuilder::setPrebuilding(bool prebuilding)
 
 ReferencedTopDUContext DeclarationBuilder::build(const IndexedString& url, Ast* node, ReferencedTopDUContext updateContext)
 {
+    m_correctionHelper.reset(new CorrectionHelper(url, this));
+
     // The declaration builder needs to run twice, so it can resolve uses of e.g. functions
     // which are called before they are defined (which is easily possible, due to python's dynamic nature).
     if ( ! m_prebuilding ) {
@@ -215,7 +218,6 @@ template<typename T> QList<Declaration*> DeclarationBuilder::reopenFittingDeclar
         }
         if ( fitting && ! reallyEncountered && ! invalidType ) {
             if ( d->topContext() == currentContext()->topContext() ) {
-                kDebug() << "Opening previously existing declaration for " << d->toString();
                 openDeclarationInternal(d);
                 d->setRange(updateRangeTo);
                 *ok = d;
@@ -240,13 +242,16 @@ template<typename T> T* DeclarationBuilder::visitVariableDeclaration(Identifier*
     Ast* rangeNode = originalAst ? originalAst : node;
     RangeInRevision range = editorFindRange(rangeNode, rangeNode);
     
+    // ask the correction file library if there's a user-specified type for this object
+    if ( AbstractType::Ptr hint = m_correctionHelper->hintForLocal(node->value) ) {
+        type = hint;
+    }
+
     // If no type is known, display "mixed".
     if ( ! type ) {
         type = AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed));
     }
     
-    kDebug() << "Parsing variable declaration: " << node->value;
-    
     QList<Declaration*> existingDeclarations;
     if ( previous ) {
         existingDeclarations << previous;
@@ -499,6 +504,20 @@ Declaration* DeclarationBuilder::findDeclarationInContext(QStringList dottedName
     return lastAccessedDeclaration;
 }
 
+QString DeclarationBuilder::buildModuleNameFromNode(ImportFromAst* node, AliasAst* alias, const QString& intermediate) {
+    QString moduleName = alias->name->value;
+    if ( ! intermediate.isEmpty() ) {
+        moduleName.prepend('.').prepend(intermediate);
+    }
+    if ( node->module ) {
+        moduleName.prepend('.').prepend(node->module->value);
+    }
+    // To handle relative import correct, add node level in the beginning of the path
+    // This will allow findModulePath to deduce module search direcotry properly
+    moduleName.prepend(QString(node->level, '.'));
+    return moduleName;
+}
+
 void DeclarationBuilder::visitImportFrom(ImportFromAst* node)
 {
     Python::AstDefaultVisitor::visitImportFrom(node);
@@ -506,12 +525,6 @@ void DeclarationBuilder::visitImportFrom(ImportFromAst* node)
     QString declarationName;
     foreach ( AliasAst* name, node->names ) {
         // iterate over all the names that are imported, like "from foo import bar as baz, bang as asdf"
-        if ( node->module ) {
-            moduleName = node->module->value + '.' + name->name->value;
-        }
-        else {
-            moduleName = '.' + name->name->value;
-        }
         Identifier* declarationIdentifier = 0;
         declarationName.clear();
         if ( name->asName ) {
@@ -526,10 +539,19 @@ void DeclarationBuilder::visitImportFrom(ImportFromAst* node)
         // This is a bit hackish, it tries to find the specified object twice twice -- once it tries to
         // import the name from a module's __init__.py file, and once from a "real" python file
         // TODO improve this code-wise
-        Declaration* success = createModuleImportDeclaration(moduleName, declarationName, declarationIdentifier, 0, DontCreateProblems);
-        if ( not success and node->module ) {
-            QString modifiedModuleName = node->module->value + ".__init__." + name->name->value;
-            createModuleImportDeclaration(modifiedModuleName, declarationName, declarationIdentifier);
+        ProblemPointer problem(0);
+        QString intermediate;
+        moduleName = buildModuleNameFromNode(node, name, intermediate);
+        Declaration* success = createModuleImportDeclaration(moduleName, declarationName, declarationIdentifier, problem);
+        if ( ! success && (node->module || node->level) ) {
+            ProblemPointer problem_init(0);
+            intermediate = QString("__init__");
+            moduleName = buildModuleNameFromNode(node, name, intermediate);
+            success = createModuleImportDeclaration(moduleName, declarationName, declarationIdentifier, problem_init);
+        }
+        if ( ! success && problem ) {
+            DUChainWriteLocker lock;
+            topContext()->addProblem(problem);
         }
     }
 }
@@ -537,7 +559,6 @@ void DeclarationBuilder::visitImportFrom(ImportFromAst* node)
 void DeclarationBuilder::visitComprehension(ComprehensionAst* node)
 {
     Python::AstDefaultVisitor::visitComprehension(node);
-    kDebug() << "visiting comprehension" << currentContext()->range();
     // make the declaration zero chars long; it must appear at the beginning of the context,
     // because it is actually used *before* its real declaration: [foo for foo in bar]
     // The DUChain doesn't like this, so for now, the declaration is at the opening bracket,
@@ -588,7 +609,12 @@ void DeclarationBuilder::visitImport(ImportAst* node)
         QString moduleName = name->name->value;
         // use alias if available, name otherwise
         Identifier* declarationIdentifier = name->asName ? name->asName : name->name;
-        createModuleImportDeclaration(moduleName, declarationIdentifier->value, declarationIdentifier);
+        ProblemPointer problem(0);
+        createModuleImportDeclaration(moduleName, declarationIdentifier->value, declarationIdentifier, problem);
+        if ( problem ) {
+            DUChainWriteLocker lock;
+            topContext()->addProblem(problem);
+        }
     }
 }
 
@@ -770,10 +796,10 @@ Declaration* DeclarationBuilder::createDeclarationTree(const QStringList& nameCo
 
 Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleName, QString declarationName,
                                                                Identifier* declarationIdentifier,
-                                                               Ast* rangeNode, ProblemPolicy createProblem)
+                                                               ProblemPointer& problemEncountered, Ast* rangeNode)
 {
     // Search the disk for a python file which contains the requested declaration
-    QPair<KUrl, QStringList> moduleInfo = findModulePath(moduleName);
+    QPair<KUrl, QStringList> moduleInfo = findModulePath(moduleName, currentlyParsedDocument().toUrl());
     kDebug() << moduleName;
     RangeInRevision range(RangeInRevision::invalid());
     if ( rangeNode ) {
@@ -796,51 +822,25 @@ Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleNam
         // a missing module, or a C module (.so) which is unreadable for kdevelop
         // TODO imrpove error handling in case the module exists as a shared object or .pyc file only
         kDebug() << "invalid or non-existent URL:" << moduleInfo;
-        if ( createProblem != DontCreateProblems ) {
-            KDevelop::Problem *p = new KDevelop::Problem();
-            p->setFinalLocation(DocumentRange(currentlyParsedDocument(), range.castToSimpleRange())); // TODO ok?
-            p->setSource(KDevelop::ProblemData::SemanticAnalysis);
-            p->setSeverity(KDevelop::ProblemData::Warning);
-            p->setDescription(i18n("Module \"%1\" not found", moduleName));
-            {
-                DUChainWriteLocker wlock(DUChain::lock());
-                ProblemPointer ptr(p);
-                topContext()->addProblem(ptr);
-            }
-        }
+        KDevelop::Problem *p = new Python::MissingIncludeProblem(moduleName, currentlyParsedDocument());
+        p->setFinalLocation(DocumentRange(currentlyParsedDocument(), range.castToSimpleRange()));
+        p->setSource(KDevelop::ProblemData::SemanticAnalysis);
+        p->setSeverity(KDevelop::ProblemData::Warning);
+        p->setDescription(i18n("Module \"%1\" not found", moduleName));
+        problemEncountered.attach(p);
         return 0;
     }
     if ( ! moduleContext ) {
         // schedule the include file for parsing, and schedule the current one for reparsing after that is done
         kDebug() << "No module context, recompiling";
         m_unresolvedImports.append(modulePath);
-        BackgroundParser* bgparser = KDevelop::ICore::self()->languageController()->backgroundParser();
-        bool needsReschedule = true;
-        if ( bgparser->isQueued(modulePath) ) {
-            const ParseJob* job = bgparser->parseJobForDocument(modulePath);
-            int previousPriority = BackgroundParser::WorstPriority;
-            if ( job ) {
-                previousPriority = job->parsePriority();
-            }
-            // if it's less important, reschedule it
-            if ( job && previousPriority > m_ownPriority - 1 ) {
-                bgparser->removeDocument(modulePath);
-            }
-            else if ( job ) {
-                needsReschedule = false;
-            }
-        }
-        if ( needsReschedule ) {
-            bgparser->addDocument(modulePath, TopDUContext::ForceUpdate, m_ownPriority - 1,
-                                  0, ParseJob::FullSequentialProcessing);
-        }
+        Helper::scheduleDependency(modulePath, m_ownPriority);
         // parseDocuments() must *not* be called from a background thread!
         // KDevelop::ICore::self()->languageController()->backgroundParser()->parseDocuments();
         return 0;
     }
     if ( moduleInfo.second.isEmpty() ) {
         // import the whole module
-        kDebug() << "Got module, importing it" << declarationIdentifier->value;
         resultingDeclaration = createDeclarationTree(declarationName.split("."),
                                                      declarationIdentifier, moduleContext, 0, range);
     }
@@ -854,21 +854,19 @@ Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleNam
         else {
             kDebug() << "Got module, importing declaration: " << moduleInfo.second;
             Declaration* originalDeclaration = findDeclarationInContext(moduleInfo.second, moduleContext);
-            kDebug() << "Result: " << originalDeclaration;
             if ( originalDeclaration ) {
                 DUChainWriteLocker lock(DUChain::lock());
                 resultingDeclaration = createDeclarationTree(declarationName.split("."), declarationIdentifier,
                                                              ReferencedTopDUContext(0), originalDeclaration,
                                                              editorFindRange(declarationIdentifier, declarationIdentifier));
             }
-            else if ( createProblem != DontCreateProblems ) {
-                KDevelop::Problem *p = new KDevelop::Problem();
+            else {
+                KDevelop::Problem *p = new Python::MissingIncludeProblem(moduleName, currentlyParsedDocument());
                 p->setFinalLocation(DocumentRange(currentlyParsedDocument(), range.castToSimpleRange())); // TODO ok?
                 p->setSource(KDevelop::ProblemData::SemanticAnalysis);
                 p->setSeverity(KDevelop::ProblemData::Warning);
                 p->setDescription(i18n("Declaration for \"%1\" not found in specified module", moduleInfo.second.join(".")));
-                ProblemPointer ptr(p);
-                topContext()->addProblem(ptr);
+                problemEncountered.attach(p);
             }
         }
     }
@@ -929,7 +927,6 @@ void DeclarationBuilder::visitLambda(LambdaAst* node)
 void DeclarationBuilder::visitCall(CallAst* node)
 {
     Python::AstDefaultVisitor::visitCall(node);
-    
     // Find the function being called; this code also handles cases where non-names
     // are called, for example:
     //     class myclass():
@@ -939,14 +936,13 @@ void DeclarationBuilder::visitCall(CallAst* node)
     // In the above example, this call will be evaluated to "myclass.myfun" in the following block.
     ExpressionVisitor functionVisitor(currentContext(), editor());
     functionVisitor.visitNode(node);
-    
+
     if ( node->function && node->function->astType == Ast::AttributeAstType && functionVisitor.lastDeclaration() ) {
         // Some special functions, like "append", update the content of the object they operate on.
-        kDebug() << "Checking for list content updates...";
         // Find the object the function is called on, like for d = [1, 2, 3]; d.append(5), this will give "d"
         ExpressionVisitor v(currentContext(), editor());
         v.visitNode(static_cast<AttributeAst*>(node->function)->value);
-        
+
         // Don't do anything if the object the function is being called on is not a container.
         if ( VariableLengthContainer::Ptr container = v.lastType().cast<VariableLengthContainer>() ) {
             // Don't to updates to pre-defined functions.
@@ -1015,8 +1011,6 @@ void DeclarationBuilder::visitCall(CallAst* node)
     //     def foo(arg): print arg
     //     foo(3)
     // the following will change the type of "arg" to be "int" when it processes the second line.
-    kDebug() << "--";
-    kDebug() << "Trying to update function argument types based on call";
     
     DUChainReadLocker lock;
     QPair<FunctionDeclaration::Ptr, bool> lastFunctionDeclarationP = Helper::functionDeclarationForCalledDeclaration(functionVisitor.lastDeclaration());
@@ -1045,7 +1039,7 @@ void DeclarationBuilder::visitCall(CallAst* node)
                     parameters.remove(0);
                 }
             }
-            
+
             int atParam = 0;
             bool atVararg = false;
             // Check that there's enough known parameters which can be updated
@@ -1064,10 +1058,11 @@ void DeclarationBuilder::visitCall(CallAst* node)
                             break;
                         }
                     }
-                    
+
                     // Get the type of the argument
                     ExpressionVisitor argumentVisitor(currentContext(), editor());
                     argumentVisitor.visitNode(arg);
+
                     lock.unlock();
                     DUChainWriteLocker wlock;
                     if ( argumentVisitor.lastType() ) {
@@ -1111,7 +1106,7 @@ void DeclarationBuilder::visitCall(CallAst* node)
                 }
                 lock.unlock();
                 DUChainWriteLocker wlock;
-                if ( lastFunctionDeclaration->hasKwarg() ) {
+                if ( lastFunctionDeclaration->hasKwarg() && ! parameters.isEmpty() ) {
                     foreach ( KeywordAst* keyword, node->keywords ) {
                         AbstractType::Ptr param = parameters.last()->abstractType();
                         VariableLengthContainer::Ptr variable = param.cast<VariableLengthContainer>();
@@ -1141,7 +1136,6 @@ void DeclarationBuilder::visitCall(CallAst* node)
 
 void DeclarationBuilder::visitAssignment(AssignmentAst* node)
 {
-    KDEBUG_BLOCK
     AstDefaultVisitor::visitAssignment(node);
     // Because of tuple unpacking, it is required to gather the left- and right hand side
     // expressions / types first, then match them together in a second step.
@@ -1177,8 +1171,6 @@ void DeclarationBuilder::visitAssignment(AssignmentAst* node)
         rhsTypes << v.lastType();
         rhsDeclarations << v.lastDeclaration();
         rhsEntryIsAliasDeclaration << v.m_isAlias;
-        DUChainReadLocker lock;
-        kDebug() << ( v.lastType() ? v.lastType()->toString() : "< no last type >" ) << ( v.lastDeclaration() ? v.lastDeclaration()->toString() : "< no last declaration >" );
     }
     
     // Now all the information about left- and right hand side entries is ready,
@@ -1259,7 +1251,7 @@ void DeclarationBuilder::visitAssignment(AssignmentAst* node)
             if ( tupleElementType ) {
                 ExpressionVisitor targetVisitor(currentContext());
                 targetVisitor.visitNode(v);
-                DUChainReadLocker lock(DUChain::lock());
+                DUChainWriteLocker lock;
                 VariableLengthContainer::Ptr cont = VariableLengthContainer::Ptr::dynamicCast(targetVisitor.lastType());
                 if ( cont ) {
                     cont->addContentType(tupleElementType);
@@ -1276,9 +1268,8 @@ void DeclarationBuilder::visitAssignment(AssignmentAst* node)
                         }
                     }
                 }
-                lock.unlock();
-                if ( DeclarationPointer lastDecl = targetVisitor.lastDeclaration() ) {
-                    DUChainWriteLocker wlock(DUChain::lock());
+                DeclarationPointer lastDecl = targetVisitor.lastDeclaration();
+                if ( cont && lastDecl ) {
                     lastDecl->setAbstractType(cont.cast<AbstractType>());
                 }
             }
@@ -1360,6 +1351,8 @@ void DeclarationBuilder::visitAssignment(AssignmentAst* node)
 
 void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
 {
+    const CorrectionHelper::Recursion r(m_correctionHelper->enterClass(node->name->value));
+
     StructureType::Ptr type(new StructureType());
     
     DUChainWriteLocker lock(DUChain::lock());
@@ -1403,12 +1396,12 @@ void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
     lock.lock();
     // every python class inherits from "object".
     // We use this to add all the __str__, __get__, ... methods.
-    if ( dec->baseClassesSize() == 0 and node->name->value != "__kdevpythondocumentation_builtin_object" ) {
+    if ( dec->baseClassesSize() == 0 and node->name->value != "object" ) {
         DUChainWriteLocker wlock;
         ReferencedTopDUContext docContext = Helper::getDocumentationFileContext();
         if ( docContext ) {
             QList<Declaration*> object = docContext->findDeclarations(
-                QualifiedIdentifier("__kdevpythondocumentation_builtin_object")
+                QualifiedIdentifier("object")
             );
             if ( ! object.isEmpty() && object.first()->abstractType() ) {
                 Declaration* objDecl = object.first();
@@ -1474,13 +1467,14 @@ template<typename T> void DeclarationBuilder::visitDecorators(QList< Python::Exp
 
 void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
 {
+    const CorrectionHelper::Recursion r(m_correctionHelper->enterFunction(node->name->value));
+
     // Search for an eventual containing class declaration;
     // if that exists, then this function is a member function
     DeclarationPointer eventualParentDeclaration(currentDeclaration());
     FunctionType::Ptr type(new FunctionType());
     
     DUChainWriteLocker lock(DUChain::lock());
-    kDebug() << identifierForNode(node->name).toString();
     FunctionDeclaration* dec = eventuallyReopenDeclaration<FunctionDeclaration>(node->name, node->name, FunctionDeclarationType);
     
     Q_ASSERT(dec->isFunctionDeclaration());
@@ -1520,26 +1514,44 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
     }
     
     {
+        static IndexedString constructorName("__init__");
         DUChainWriteLocker lock(DUChain::lock());
+        if ( dec->identifier().identifier() == constructorName ) {
+            // the constructor returns an instance of the object,
+            // nice to display it in tooltips etc.
+            type->setReturnType(currentType<AbstractType>());
+        }
         if ( ! type->returnType() ) {
             type->setReturnType(AbstractType::Ptr(new IntegralType(IntegralType::TypeVoid)));
         }
         dec->setType(type);
     }
-    
+
     if ( ! isStatic ) {
         DUContext* args = DUChainUtils::getArgumentContext(dec);
         if ( args )  {
             QVector<Declaration*> parameters = args->localDeclarations();
-            
+            static IndexedString newMethodName("__new__");
+            static IndexedString selfArgumentName("self");
+            static IndexedString clsArgumentName("cls");
             if ( currentContext()->type() == DUContext::Class && ! parameters.isEmpty() && ! isClassMethod ) {
-                if ( parameters[0]->identifier().identifier() != IndexedString("self") ) {
-                    kDebug() << "argument is not called self, but instead:" << parameters[0]->identifier().identifier().str();
+                QString description;
+                if ( dec->identifier().identifier() == newMethodName
+                     && parameters[0]->identifier().identifier() != clsArgumentName )
+                {
+                    description = i18n("First argument of __new__ method is not called cls, this is deprecated");
+                }
+                else if ( dec->identifier().identifier() != newMethodName
+                          && parameters[0]->identifier().identifier() != selfArgumentName )
+                {
+                    description = i18n("First argument of class method is not called self, this is deprecated");
+                }
+                if ( ! description.isEmpty() ) {
                     KDevelop::Problem *p = new KDevelop::Problem();
+                    p->setDescription(description);
                     p->setFinalLocation(DocumentRange(currentlyParsedDocument(), parameters[0]->range().castToSimpleRange()));
                     p->setSource(KDevelop::ProblemData::SemanticAnalysis);
                     p->setSeverity(KDevelop::ProblemData::Warning);
-                    p->setDescription(i18n("First argument of class method is not called self, this is deprecated"));
                     ProblemPointer ptr(p);
                     topContext()->addProblem(ptr);
                 }
@@ -1548,7 +1560,8 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
             else if ( currentContext()->type() == DUContext::Class && parameters.isEmpty() ) {
                 DUChainWriteLocker lock(DUChain::lock());
                 KDevelop::Problem *p = new KDevelop::Problem();
-                p->setFinalLocation(DocumentRange(currentlyParsedDocument(), SimpleRange(node->startLine, node->startCol, node->startLine, 10000))); // only mark first line
+                 // only mark first line
+                p->setFinalLocation(DocumentRange(currentlyParsedDocument(), SimpleRange(node->startLine, node->startCol, node->startLine, 10000)));
                 p->setSource(KDevelop::ProblemData::SemanticAnalysis);
                 p->setSeverity(KDevelop::ProblemData::Warning);
                 p->setDescription(i18n("Non-static class method without arguments, must have at least one (self)"));
@@ -1561,6 +1574,11 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
         m_firstAttributeDeclaration = DeclarationPointer(0);
         dec->setStatic(true);
     }
+
+    if ( AbstractType::Ptr hint = m_correctionHelper->returnTypeHint() ) {
+        type->setReturnType(hint);
+        dec->setType<FunctionType>(type);
+    }
     
     // check for documentation
     dec->setComment(getDocstring(node->body));
@@ -1579,6 +1597,102 @@ QString DeclarationBuilder::getDocstring(QList< Ast* > body)
     return QString();
 }
 
+void DeclarationBuilder::visitAssertion(AssertionAst* node)
+{
+    adjustForTypecheck(node->condition, false);
+    Python::AstDefaultVisitor::visitAssertion(node);
+}
+
+void DeclarationBuilder::visitIf(IfAst* node)
+{
+    adjustForTypecheck(node->condition, true);
+    Python::AstDefaultVisitor::visitIf(node);
+}
+
+void DeclarationBuilder::adjustForTypecheck(Python::ExpressionAst* check, bool useUnsure)
+{
+    if ( ! check ) return;
+    if ( check->astType == Ast::UnaryOperationAstType
+         && static_cast<UnaryOperationAst*>(check)->type == Ast::UnaryOperatorNot )
+    {
+        // It could be something like " if not isinstance(foo, Bar): return None ".
+        check = static_cast<UnaryOperationAst*>(check)->operand;
+    }
+    if ( check->astType == Ast::CallAstType ) {
+        // Is this a call of the form "isinstance(foo, bar)"?
+        CallAst* call = static_cast<CallAst*>(check);
+        if ( ! call->function ) {
+            return;
+        }
+        if ( call->function->astType != Ast::NameAstType ) {
+            return;
+        }
+        const QString functionName = static_cast<Python::NameAst*>(call->function)->identifier->value;
+        if ( functionName != QLatin1String("isinstance") ) {
+            return;
+        }
+        if ( call->arguments.length() != 2 ) {
+            return;
+        }
+        adjustExpressionsForTypecheck(call->arguments.at(0), call->arguments.at(1), useUnsure);
+    }
+    else if ( check->astType == Ast::CompareAstType ) {
+        // Is this a call of the form "type(ainstance) == a"?
+        CompareAst* compare = static_cast<CompareAst*>(check);
+        if ( compare->operators.size() != 1 || compare->comparands.size() != 1 ) {
+            return;
+        }
+        if ( compare->operators.first() != Ast::ComparisonOperatorEquals ) {
+            return;
+        }
+        ExpressionAst* c1 = compare->comparands.first();
+        ExpressionAst* c2 = compare->leftmostElement;
+        if ( ! ( (c1->astType == Ast::CallAstType) ^ (c2->astType == Ast::CallAstType) ) ) {
+            // Exactly one of the two must be a call. TODO: support adjusting function return types
+            return;
+        }
+        CallAst* typecall = static_cast<CallAst*>(c1->astType == Ast::CallAstType ? c1 : c2);
+        if ( ! typecall->function || typecall->function->astType != Ast::NameAstType || typecall->arguments.length() != 1 ) {
+            return;
+        }
+        const QString functionName = static_cast<Python::NameAst*>(typecall->function)->identifier->value;
+        if ( functionName != QLatin1String("type") ) {
+            return;
+        }
+        adjustExpressionsForTypecheck(typecall->arguments.at(0), c1->astType == Ast::CallAstType ? c2 : c1, useUnsure);
+    }
+}
+
+void DeclarationBuilder::adjustExpressionsForTypecheck(Python::ExpressionAst* adjustExpr, Python::ExpressionAst* from, bool useUnsure)
+{
+    // Find types of the two arguments
+    ExpressionVisitor first(currentContext());
+    ExpressionVisitor second(currentContext());
+    first.visitNode(adjustExpr);
+    second.visitNode(from);
+    AbstractType::Ptr hint;
+    DeclarationPointer adjust;
+    if ( second.m_isAlias && second.lastType() ) {
+        hint = second.lastType();
+        adjust = first.lastDeclaration();
+    }
+    if ( ! adjust || adjust->isFunctionDeclaration() ) {
+        // no declaration for the thing to verify, can't adjust it.
+        return;
+    }
+    else if ( adjust->topContext() == Helper::getDocumentationFileContext() ) {
+        // do not motify types in the doc context
+        return;
+    }
+    DUChainWriteLocker lock;
+    if ( useUnsure ) {
+        adjust->setAbstractType(Helper::mergeTypes(adjust->abstractType(), hint));
+    }
+    else {
+        adjust->setAbstractType(hint);
+    }
+}
+
 void DeclarationBuilder::visitReturn(ReturnAst* node)
 {
     // Find the type of the object being "return"ed
@@ -1609,7 +1723,6 @@ void DeclarationBuilder::visitReturn(ReturnAst* node)
 void DeclarationBuilder::visitArguments( ArgumentsAst* node )
 {
     DUChainWriteLocker lock(DUChain::lock());
-    kDebug() << "Current context for parameters: " << currentContext() << currentContext()->scopeIdentifier().toString();
     
     if ( currentDeclaration() and currentDeclaration()->isFunctionDeclaration() ) {
         FunctionDeclaration* workingOnDeclaration = static_cast<FunctionDeclaration*>(Helper::resolveAliasDeclaration(currentDeclaration()));
@@ -1622,12 +1735,11 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
             int parametersCount = node->arguments.length();
             int firstDefaultParameterOffset = parametersCount - defaultParametersCount;
             int currentIndex = 0;
-            kDebug() << "variable argument ranges: " << node->arg_lineno << node->arg_col_offset << node->vararg_lineno << node->vararg_col_offset;
             foreach ( ExpressionAst* expression, node->arguments ) {
                 // Iterate over all the function's arguments, create declarations, and add the arguments
                 // to the functions FunctionType.
                 currentIndex += 1;
-                realParam = dynamic_cast<NameAst*>(expression);
+                realParam = (expression && expression->astType == Ast::NameAstType) ? static_cast<NameAst*>(expression) : 0;
                 
                 if ( ! realParam || realParam->context != ExpressionAst::Parameter ) {
                     // def myfun(x, y, (z, a, b), d): pass
@@ -1640,7 +1752,6 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
                 
                 if ( type && paramDeclaration && currentIndex > firstDefaultParameterOffset ) {
                     // Handle arguments with default values, like def foo(bar = 3): pass
-                    kDebug() << "Adding default argument: " << realParam->identifier->value << paramDeclaration->abstractType();
                     // Find type of given default value, and assign it to the declaration
                     // TODO does this actually work?
                     ExpressionVisitor v(currentContext());
@@ -1655,10 +1766,8 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
                     }
                     // TODO add the real expression from the document here as default value
                     workingOnDeclaration->addDefaultParameter(IndexedString("..."));
-                    kDebug() << "Arguments count: " << type->arguments().length();
                 }
                 else {
-                    kDebug() << "Not a default argument: " << realParam->identifier->value;
                     // For now, we cannot know the type, thus we write "mixed".
                     // As soon as a call to the function is encountered, this type might be updated.
                     DUChainWriteLocker lock;
@@ -1672,7 +1781,6 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
                 }
             }
             // Handle *args, **kwargs, and assign them a list / dictionary type.
-            kDebug() << "var/kwarg:" <<  node->vararg << node->kwarg;
             if ( node->vararg ) {
                 DUChainReadLocker lock;
                 IndexedContainer::Ptr tupleType = ExpressionVisitor::typeObjectForIntegralType
@@ -1687,7 +1795,7 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
             if ( node->kwarg ) {
                 DUChainReadLocker lock;
                 AbstractType::Ptr stringType = ExpressionVisitor::typeObjectForIntegralType
-                                                                  <AbstractType>("string", currentContext());
+                                                                <AbstractType>("str", currentContext());
                 VariableLengthContainer::Ptr dictType = ExpressionVisitor::typeObjectForIntegralType
                                                                   <VariableLengthContainer>("dict", currentContext());
                 if ( dictType && stringType ) {
@@ -1706,6 +1814,31 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
     DeclarationBuilderBase::visitArguments(node);
 }
 
+void DeclarationBuilder::visitGlobal(GlobalAst* node)
+{
+    TopDUContext* top = topContext();
+    foreach ( Identifier *id, node->names ) {
+        QualifiedIdentifier qid = identifierForNode(id);
+        DUChainWriteLocker lock;
+        QList< Declaration* > existing = top->findLocalDeclarations(qid.first());
+        if ( ! existing.empty() ) {
+            AliasDeclaration* ndec = openDeclaration<AliasDeclaration>(id, node);
+            ndec->setAliasedDeclaration(existing.first());
+            closeDeclaration();
+        }
+        else {
+            injectContext(top);
+            Declaration* dec = visitVariableDeclaration<Declaration>(id);
+            dec->setRange(editorFindRange(id, id));
+            dec->setAutoDeclaration(true);
+            closeContext();
+            AliasDeclaration* ndec = openDeclaration<AliasDeclaration>(id, node);
+            ndec->setAliasedDeclaration(dec);
+            closeDeclaration();
+        }
+    }
+}
+
 }
 
 
