@@ -50,44 +50,6 @@ namespace Python
 {
     
 typedef DUChainPointer<FunctionDeclaration> FunctionDeclarationPointer;
-    
-typedef KDevelop::IntegralTypeData IntegralTypeExtendedData;
-class KDEVPYTHONDUCHAIN_EXPORT IntegralTypeExtended : public KDevelop::IntegralType {
-public:
-    typedef TypePtr<IntegralTypeExtended> Ptr;
-    enum PythonIntegralTypes {
-        TypeList = KDevelop::IntegralType::TypeLanguageSpecific,
-        TypeDict = KDevelop::IntegralType::TypeLanguageSpecific + 1
-    };
-    IntegralTypeExtended(uint type = TypeNone) : IntegralType(createData<IntegralTypeExtended>()) {
-        setDataType(type);
-        setModifiers(ConstModifier);
-    };
-    IntegralTypeExtended(IntegralTypeExtendedData& data) : IntegralType(data) { }
-    
-    virtual QString toString() const {
-        switch ( d_func()->m_dataType ) {
-            case TypeList: return i18n("list");
-            case TypeDict: return i18n("dictionary");
-            default: break;
-        }
-        
-        return KDevelop::IntegralType::toString();
-    };
-    
-    enum {
-        Identity = 60 // TODO ok?
-    };
-    
-    typedef KDevelop::IntegralTypeData Data;
-    typedef KDevelop::IntegralType BaseType;
-protected:
-    TYPE_DECLARE_DATA(IntegralTypeExtended);
-};
-
-/**
- * @brief The DUChain must be read-locked in order to run the visitor.
- **/
 
 class KDEVPYTHONDUCHAIN_EXPORT ExpressionVisitor : public AstDefaultVisitor
 {
@@ -173,6 +135,7 @@ class KDEVPYTHONDUCHAIN_EXPORT ExpressionVisitor : public AstDefaultVisitor
         };
         
         void encounter(AbstractType::Ptr type, EncounterFlags flags = (EncounterFlags) 0);
+        AbstractType::Ptr fromBinaryOperator(AbstractType::Ptr lhs, AbstractType::Ptr rhs, const QString& op);
         AbstractType::Ptr encounterPreprocess(AbstractType::Ptr type, bool merge = false);
         template<typename T> void encounter(TypePtr<T> type, EncounterFlags flags = (EncounterFlags) 0);
         void encounterDeclaration(DeclarationPointer ptr, bool isAlias = false);
