@@ -1,26 +1,21 @@
 /*****************************************************************************
  * This file is part of KDevelop                                             *
- * Copyright 2011-2012 Sven Brauch <svenbrauch@googlemail.com>               *
+ * Copyright 2011-2013 Sven Brauch <svenbrauch@googlemail.com>               *
  *                                                                           *
- * Permission is hereby granted, free of charge, to any person obtaining     *
- * a copy of this software and associated documentation files (the           *
- * "Software"), to deal in the Software without restriction, including       *
- * without limitation the rights to use, copy, modify, merge, publish,       *
- * distribute, sublicense, and/or sell copies of the Software, and to        *
- * permit persons to whom the Software is furnished to do so, subject to     *
- * the following conditions:                                                 *
+ * This program is free software; you can redistribute it and/or             *
+ * modify it under the terms of the GNU General Public License as            *
+ * published by the Free Software Foundation; either version 2 of            *
+ * the License, or (at your option) any later version.                       *
  *                                                                           *
- * The above copyright notice and this permission notice shall be            *
- * included in all copies or substantial portions of the Software.           *
+ * This program is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
+ * GNU General Public License for more details.                              *
  *                                                                           *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,           *
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF        *
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                     *
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE    *
- * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION    *
- * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION     *
- * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.           *
- *****************************************************************************/
+ * You should have received a copy of the GNU General Public License         *
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
+ *****************************************************************************
+ */
 
 #include "helpers.h"
 
@@ -32,28 +27,37 @@
 
 #include <language/duchain/types/unsuretype.h>
 #include <language/duchain/types/integraltype.h>
+#include <language/duchain/duchainutils.h>
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/duchain.h>
 #include <language/duchain/classdeclaration.h>
 #include <language/duchain/aliasdeclaration.h>
+#include <language/backgroundparser/backgroundparser.h>
 #include <interfaces/iproject.h>
 #include <interfaces/icore.h>
 #include <interfaces/iprojectcontroller.h>
+#include <interfaces/ilanguagecontroller.h>
+#include <interfaces/idocumentcontroller.h>
+
+#include <KTextEditor/View>
 
 #include "ast.h"
 #include "types/hintedtype.h"
 #include "types/unsuretype.h"
 #include "types/variablelengthcontainer.h"
 #include "types/indexedcontainer.h"
+#include "kdevpythonversion.h"
 
 using namespace KDevelop;
 
 namespace Python {
 
 QList<KUrl> Helper::cachedSearchPaths;
-QString Helper::dataDir;
+QStringList Helper::dataDirs;
 QString Helper::documentationFile;
 DUChainPointer<TopDUContext> Helper::documentationFileContext = DUChainPointer<TopDUContext>(0);
+QStringList Helper::correctionFileDirs;
+QString Helper::localCorrectionFileDir;
 
 AbstractType::Ptr Helper::resolveType(AbstractType::Ptr type)
 {
@@ -64,28 +68,74 @@ AbstractType::Ptr Helper::resolveType(AbstractType::Ptr type)
         return type;
 }
 
+void Helper::scheduleDependency(const IndexedString& dependency, int betterThanPriority)
+{
+    BackgroundParser* bgparser = KDevelop::ICore::self()->languageController()->backgroundParser();
+    bool needsReschedule = true;
+    if ( bgparser->isQueued(dependency) ) {
+        const ParseJob* job = bgparser->parseJobForDocument(dependency);
+        int previousPriority = BackgroundParser::WorstPriority;
+        if ( job ) {
+            previousPriority = job->parsePriority();
+        }
+        // if it's less important, reschedule it
+        if ( job && previousPriority > betterThanPriority - 1 ) {
+            bgparser->removeDocument(dependency);
+        }
+        else if ( job ) {
+            needsReschedule = false;
+        }
+    }
+    if ( needsReschedule ) {
+        bgparser->addDocument(dependency, TopDUContext::ForceUpdate, betterThanPriority - 1,
+                              0, ParseJob::FullSequentialProcessing);
+    }
+}
+
+IndexedDeclaration Helper::declarationUnderCursor(bool allowUse)
+{
+    KDevelop::IDocument* doc = ICore::self()->documentController()->activeDocument();
+    if ( doc && doc->textDocument() && doc->textDocument()->activeView() ) {
+        DUChainReadLocker lock;
+        if ( allowUse ) {
+            return DUChainUtils::itemUnderCursor(doc->url(), SimpleCursor(doc->textDocument()->activeView()->cursorPosition()));
+        }
+        else {
+            return DUChainUtils::declarationInLine(SimpleCursor(doc->textDocument()->activeView()->cursorPosition()), DUChainUtils::standardContextForUrl(doc->url()));
+        }
+    }
+
+    return KDevelop::IndexedDeclaration();
+}
+
+Declaration* Helper::accessAttribute(Declaration* accessed, const QString& attribute, DUContext* current)
+{
+    if ( ! accessed || ! accessed->type<StructureType>() ) {
+        return 0;
+    }
+    StructureType::Ptr type = accessed->type<StructureType>();
+    DUChainReadLocker lock(DUChain::lock());
+    QList<DUContext*> searchContexts = Helper::internalContextsForClass(type, current->topContext());
+    foreach ( DUContext* c, searchContexts ) {
+        QList< Declaration* > found = c->findLocalDeclarations(KDevelop::Identifier(attribute));
+        if ( ! found.isEmpty() ) {
+            return found.first();
+        }
+    }
+    return 0;
+}
+
 AbstractType::Ptr Helper::extractTypeHints(AbstractType::Ptr type, TopDUContext* current)
 {
-    if ( type ) {
-        kDebug() << type->toString();
-    }
-    else {
-        kDebug();
-    }
     UnsureType::Ptr result(new UnsureType());
     unsigned short maxHints = 7;
     if ( HintedType::Ptr hinted = type.cast<HintedType>() ) {
         if ( hinted->isValid(current) && isUsefulType(hinted.cast<AbstractType>()) ) {
-            kDebug() << "Adding type hint: " << hinted->toString();
             result->addType(type->indexed());
         }
-        else {
-            kDebug() << "Discarding type hint: " << hinted->toString();
-        }
     }
     else if ( UnsureType::Ptr unsure = type.cast<UnsureType>() ) {
         int len = unsure->typesSize();
-        kDebug() << "Extracting hints from " << len << "types";
         for ( int i = 0; i < len and i < maxHints; i++ ) {
             if ( HintedType::Ptr hinted = unsure->types()[i].abstractType().cast<HintedType>() ) {
                 if ( hinted->isValid(current) ) {
@@ -156,17 +206,12 @@ QPair<FunctionDeclarationPointer, bool> Helper::functionDeclarationForCalledDecl
     DeclarationPointer lastCalledDeclaration = ptr;
     if ( lastCalledDeclaration and not lastCalledDeclaration->isFunctionDeclaration() )
     {
-        kDebug() << "No function declaration, looking for class constructor";
-        kDebug() << "Class declaration: " << lastCalledDeclaration;
         if ( lastCalledDeclaration && lastCalledDeclaration->internalContext() ) {
-            kDebug() << "ok, looking for constructor";
             QList<Declaration*> constructors = lastCalledDeclaration->internalContext()
                                                ->findDeclarations(KDevelop::Identifier("__init__"));
-            kDebug() << "Found constructors: " << constructors;
             if ( ! constructors.isEmpty() ) {
                 lastCalledDeclaration = dynamic_cast<FunctionDeclaration*>(constructors.first());
                 isConstructor = true;
-                kDebug() << "new function declaration: " << lastCalledDeclaration;
             }
         }
     }
@@ -178,12 +223,11 @@ QPair<FunctionDeclarationPointer, bool> Helper::functionDeclarationForCalledDecl
     return QPair<FunctionDeclarationPointer, bool>(lastFunctionDeclaration, isConstructor);
 }
 
-Declaration* Helper::declarationForName(NameAst* /*ast*/, const QualifiedIdentifier& identifier, const RangeInRevision& nodeRange, DUContextPointer context)
+Declaration* Helper::declarationForName(const QualifiedIdentifier& identifier, const RangeInRevision& nodeRange, DUContextPointer context)
 {
     QList<Declaration*> declarations;
     QList<Declaration*> localDeclarations;
     QList<Declaration*> importedLocalDeclarations;
-    kDebug() << "Finding declaration for name" << identifier.toString() << " before " << nodeRange.end << ", in context" << context->range();
     {
         DUChainReadLocker lock(DUChain::lock());
         if ( context.data() == context->topContext() and nodeRange.isValid() ) {
@@ -196,10 +240,9 @@ Declaration* Helper::declarationForName(NameAst* /*ast*/, const QualifiedIdentif
                                                            AbstractType::Ptr(0), DUContext::DontResolveAliases);
         importedLocalDeclarations = context->findDeclarations(identifier.last(), nodeRange.end);
     }
-    Declaration* declaration;
+    Declaration* declaration = 0;
     if ( localDeclarations.length() ) {
         declaration = localDeclarations.last();
-        kDebug() << "Using local declaration";
     }
     else if ( importedLocalDeclarations.length() ) {
         // don't use declarations from class decls, they must be referenced through "self.<foo>"
@@ -213,13 +256,11 @@ Declaration* Helper::declarationForName(NameAst* /*ast*/, const QualifiedIdentif
                 break;
             }
         } while ( not importedLocalDeclarations.isEmpty() );
-        kDebug() << "Using imported local declaration (i.e., argument)";
     }
-    else if ( declarations.length() ) {
+
+    if ( !declaration && declarations.length() ) {
         declaration = declarations.last();
-        kDebug() << "Using global declaration";
     }
-    else declaration = 0;
     return declaration;
 }
 
@@ -229,7 +270,9 @@ QList< DUContext* > Helper::internalContextsForClass(StructureType::Ptr klassTyp
     if ( ! klassType ) {
         return searchContexts;
     }
-    searchContexts << klassType->internalContext(context);
+    if ( auto c = klassType->internalContext(context) ) {
+        searchContexts << c;
+    }
     Declaration* decl = Helper::resolveAliasDeclaration(klassType->declaration(context));
     ClassDeclaration* klass = dynamic_cast<ClassDeclaration*>(decl);
     if ( klass ) {
@@ -251,18 +294,19 @@ Declaration* Helper::resolveAliasDeclaration(Declaration* decl)
 {
     AliasDeclaration* alias = dynamic_cast<AliasDeclaration*>(decl);
     if ( alias ) {
+        DUChainReadLocker lock;
         return alias->aliasedDeclaration().data();
     }
     else
         return decl;
 }
 
-QString Helper::getDataDir() {
-    if ( Helper::dataDir.isNull() ) {
+QStringList Helper::getDataDirs() {
+    if ( Helper::dataDirs.isEmpty() ) {
         KStandardDirs d;
-        Helper::dataDir = d.findDirs("data", "kdevpythonsupport/documentation_files").first();
+        Helper::dataDirs = d.findDirs("data", "kdevpythonsupport/documentation_files");
     }
-    return Helper::dataDir;
+    return Helper::dataDirs;
 }
 
 QString Helper::getDocumentationFile() {
@@ -285,6 +329,50 @@ ReferencedTopDUContext Helper::getDocumentationFileContext()
     }
     return ReferencedTopDUContext(0); // c++...
 }
+
+KUrl Helper::getCorrectionFile(KUrl document)
+{
+    if ( Helper::correctionFileDirs.isEmpty() ) {
+        KStandardDirs d;
+        Helper::correctionFileDirs = d.findDirs("data", "kdevpythonsupport/correction_files/");
+    }
+
+    foreach (QString correctionFileDir, correctionFileDirs) {
+        foreach ( const KUrl& basePath, Helper::getSearchPaths(KUrl()) ) {
+            if ( ! basePath.isParentOf(document) ) {
+                continue;
+            }
+            QString path = KUrl::relativePath(basePath.path(), document.path());
+            KUrl absolutePath(correctionFileDir + path);
+            absolutePath.cleanPath();
+
+            if ( QFile::exists(absolutePath.path()) ) {
+                return absolutePath;
+            }
+        }
+    }
+    return KUrl();
+}
+
+KUrl Helper::getLocalCorrectionFile(KUrl document)
+{
+    if ( Helper::localCorrectionFileDir.isNull() ) {
+        Helper::localCorrectionFileDir = KStandardDirs::locateLocal("data", "kdevpythonsupport/correction_files/");
+    }
+
+    KUrl absolutePath;
+    foreach ( const KUrl& basePath, Helper::getSearchPaths(KUrl()) ) {
+        if ( ! basePath.isParentOf(document) ) {
+            continue;
+        }
+        QString path = KUrl::relativePath(basePath.path(), document.path());
+        absolutePath = KUrl(Helper::localCorrectionFileDir + path);
+        absolutePath.cleanPath();
+
+        break;
+    }
+    return absolutePath;
+}
     
 QList<KUrl> Helper::getSearchPaths(KUrl workingOnDocument)
 {
@@ -294,16 +382,18 @@ QList<KUrl> Helper::getSearchPaths(KUrl workingOnDocument)
         searchPaths.append(KUrl(project->folder().url()));
     }
     
-    searchPaths.append(KUrl(getDataDir()));
+    foreach ( const QString& path, getDataDirs() ) {
+        searchPaths.append(KUrl(path));
+    }
     
     if ( cachedSearchPaths.isEmpty() ) {
         KStandardDirs d;
         kDebug() << "*** Gathering search paths...";
         QStringList getpath;
-        getpath << "python" << "-c" << "import sys; sys.stdout.write(':'.join(sys.path))";
+        getpath << "-c" << "import sys; sys.stdout.write(':'.join(sys.path))";
         
         QProcess python;
-        python.start("/usr/bin/env", getpath);
+        python.start(QLatin1String(PYTHON_EXECUTABLE), getpath);
         python.waitForFinished(1000);
         QByteArray pythonpath = python.readAllStandardOutput();
         QList<QByteArray> paths = pythonpath.split(':');
@@ -332,11 +422,12 @@ QList<KUrl> Helper::getSearchPaths(KUrl workingOnDocument)
     
     searchPaths.append(cachedSearchPaths);
     
-    // search in the current packages
-    searchPaths.append(KUrl(workingOnDocument.directory()));
+    const QString& currentDir = workingOnDocument.directory(KUrl::IgnoreTrailingSlash);
+    if ( ! currentDir.isEmpty() ) {
+        // search in the current packages
+        searchPaths.append(KUrl(currentDir));
+    }
     
-    kDebug() << "Search paths: " << searchPaths;
-    kDebug() << workingOnDocument;
     return searchPaths;
 }
 
