@@ -2,25 +2,20 @@
  * This file is part of KDevelop                                             *
  * Copyright 2011-2012 Sven Brauch <svenbrauch@googlemail.com>               *
  *                                                                           *
- * Permission is hereby granted, free of charge, to any person obtaining     *
- * a copy of this software and associated documentation files (the           *
- * "Software"), to deal in the Software without restriction, including       *
- * without limitation the rights to use, copy, modify, merge, publish,       *
- * distribute, sublicense, and/or sell copies of the Software, and to        *
- * permit persons to whom the Software is furnished to do so, subject to     *
- * the following conditions:                                                 *
+ * This program is free software; you can redistribute it and/or             *
+ * modify it under the terms of the GNU General Public License as            *
+ * published by the Free Software Foundation; either version 2 of            *
+ * the License, or (at your option) any later version.                       *
  *                                                                           *
- * The above copyright notice and this permission notice shall be            *
- * included in all copies or substantial portions of the Software.           *
+ * This program is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
+ * GNU General Public License for more details.                              *
  *                                                                           *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,           *
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF        *
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                     *
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE    *
- * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION    *
- * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION     *
- * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.           *
- *****************************************************************************/
+ * You should have received a copy of the GNU General Public License         *
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
+ *****************************************************************************
+ */
 
 #ifndef GLOBALHELPERS_H
 #define GLOBALHELPERS_H
@@ -29,6 +24,7 @@
 #include "pythonduchainexport.h"
 #include "types/unsuretype.h"
 #include "ast.h"
+#include "expressionvisitor.h"
 
 #include <interfaces/iproject.h>
 #include <language/duchain/declaration.h>
@@ -51,19 +47,30 @@ class KDEVPYTHONDUCHAIN_EXPORT Helper {
 public:
     /** get search paths for python files **/
     static QList<KUrl> getSearchPaths(KUrl workingOnDocument);
-    static QString dataDir;
+    static QStringList dataDirs;
     static QString documentationFile;
+    static QStringList correctionFileDirs;
+    static QString localCorrectionFileDir;
     static DUChainPointer<TopDUContext> documentationFileContext;
     
-    static QString getDataDir();
+    static QStringList getDataDirs();
     static QString getDocumentationFile();
     static ReferencedTopDUContext getDocumentationFileContext();
-    
+
+    static KUrl getCorrectionFile(KUrl document);
+    static KUrl getLocalCorrectionFile(KUrl document);
+
     static QList<KUrl> cachedSearchPaths;
     
     static AbstractType::Ptr extractTypeHints(AbstractType::Ptr type, TopDUContext* current);
     
     static AbstractType::Ptr resolveType(AbstractType::Ptr type);
+
+    static Declaration* accessAttribute(Declaration* accessed, const QString& attribute, DUContext* current);
+
+    static void scheduleDependency(const IndexedString& dependency, int betterThanPriority);
+
+    static KDevelop::IndexedDeclaration declarationUnderCursor(bool allowUse = true);
     
     /**
      * @brief Finds whether the specified called declaration is a function declaration, and, if not, checks for a class declaration; then returns the constructor
@@ -74,9 +81,10 @@ public:
     static QPair<FunctionDeclarationPointer, bool> functionDeclarationForCalledDeclaration(DeclarationPointer ptr);
     
     template<typename T> static const Decorator* findDecoratorByName(T* inDeclaration, const QString& name) {
-        register int count = inDeclaration->decoratorsSize();
+        int count = inDeclaration->decoratorsSize();
+        const IndexedString indexedName = IndexedString(name);
         for ( int i = 0; i < count; i++ ) {
-            if ( inDeclaration->decorators()[i].name() == name )
+            if ( inDeclaration->decorators()[i].fastName() == indexedName )
                 return &(inDeclaration->decorators()[i]);
         }
         return 0;
@@ -120,10 +128,10 @@ public:
         **/
     static Declaration* resolveAliasDeclaration(Declaration* decl);
     
-    static Declaration* declarationForName(NameAst* ast, const QualifiedIdentifier& identifier,
+    static Declaration* declarationForName(const QualifiedIdentifier& identifier,
                                            const RangeInRevision& nodeRange, DUContextPointer context);
 };
 
 }
 
-#endif
\ No newline at end of file
+#endif
