@@ -28,6 +28,7 @@
 #include <QString>
 #include <QStringList>
 #include <QPair>
+#include <QSharedPointer>
 
 #include "parserexport.h"
 
@@ -133,13 +134,13 @@ public:
     };
     
     enum BooleanOperationTypes {
-        BooleanAnd,
+        BooleanAnd = 1,
         BooleanOr,
         BooleanInvalidOperation
     };
     
     enum OperatorTypes {
-        OperatorAdd,
+        OperatorAdd = 1,
         OperatorSub,
         OperatorMult,
         OperatorDiv,
@@ -155,7 +156,7 @@ public:
     };
     
     enum UnaryOperatorTypes {
-        UnaryOperatorInvert,
+        UnaryOperatorInvert = 1,
         UnaryOperatorNot,
         UnaryOperatorAdd,
         UnaryOperatorSub,
@@ -163,7 +164,7 @@ public:
     };
     
     enum ComparisonOperatorTypes {
-        ComparisonOperatorEquals,
+        ComparisonOperatorEquals = 1,
         ComparisonOperatorNotEquals,
         ComparisonOperatorLessThan,
         ComparisonOperatorLessThanEqual,
@@ -178,7 +179,6 @@ public:
 
     Ast(Ast* parent, AstType type);
     Ast();
-    virtual ~Ast();
     Ast* parent;
     AstType astType;
     
@@ -189,10 +189,10 @@ public:
         endLine = other->endLine;
     }
 
-    qint64 startCol;
-    qint64 startLine;
-    qint64 endCol;
-    qint64 endLine;
+    int startCol;
+    int startLine;
+    int endCol;
+    int endLine;
     
     bool hasUsefulRangeInformation;
     
@@ -231,6 +231,8 @@ public:
 class KDEVPYTHONPARSER_EXPORT CodeAst : public Ast {
 public:
     CodeAst();
+    ~CodeAst();
+    typedef QSharedPointer<CodeAst> Ptr;
     QList<Ast*> body;
     Identifier* name; // module name
 };
@@ -431,6 +433,33 @@ public:
     Ast::OperatorTypes type;
     ExpressionAst* lhs;
     ExpressionAst* rhs;
+    inline QString methodName() const {
+        switch ( type ) {
+            case Python::Ast::OperatorAdd: return QLatin1String("__add__");
+            case Python::Ast::OperatorBitwiseAnd: return QLatin1String("__and__");
+            case Python::Ast::OperatorBitwiseOr: return QLatin1String("__or__");
+            case Python::Ast::OperatorBitwiseXor: return QLatin1String("__xor__");
+            case Python::Ast::OperatorDiv: return QLatin1String("__div__");
+            case Python::Ast::OperatorFloorDivision: return QLatin1String("__floordiv__");
+            case Python::Ast::OperatorLeftShift: return QLatin1String("__lshift__");
+            case Python::Ast::OperatorMod: return QLatin1String("__mod__");
+            case Python::Ast::OperatorMult: return QLatin1String("__mul__");
+            case Python::Ast::OperatorPow: return QLatin1String("__pow__");
+            case Python::Ast::OperatorRightShift: return QLatin1String("__rshift__");
+            case Python::Ast::OperatorSub: return QLatin1String("__sub__");
+            case Python::Ast::OperatorInvalid: // fallthrough
+            default: return QString();
+        }
+    };
+    // incremental methods, for e.g. a += 3
+    inline QString incMethodName() const {
+        QString name = methodName();
+        if ( name.size() < 3 ) {
+            return name;
+        }
+        name.insert(2, QLatin1Char('i'));
+        return name;
+    }
 };
 
 class KDEVPYTHONPARSER_EXPORT UnaryOperationAst : public ExpressionAst {
