@@ -2,38 +2,33 @@
  * Copyright (c) 2007 Andreas Pakulat <apaku@gmx.de>                         *
  * Copyright (c) 2007 Piyush verma <piyush.verma@gmail.com>                  *
  *                                                                           *
- * Permission is hereby granted, free of charge, to any person obtaining     *
- * a copy of this software and associated documentation files (the           *
- * "Software"), to deal in the Software without restriction, including       *
- * without limitation the rights to use, copy, modify, merge, publish,       *
- * distribute, sublicense, and/or sell copies of the Software, and to        *
- * permit persons to whom the Software is furnished to do so, subject to     *
- * the following conditions:                                                 *
+ * This program is free software; you can redistribute it and/or             *
+ * modify it under the terms of the GNU General Public License as            *
+ * published by the Free Software Foundation; either version 2 of            *
+ * the License, or (at your option) any later version.                       *
  *                                                                           *
- * The above copyright notice and this permission notice shall be            *
- * included in all copies or substantial portions of the Software.           *
+ * This program is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
+ * GNU General Public License for more details.                              *
  *                                                                           *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,           *
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF        *
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                     *
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE    *
- * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION    *
- * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION     *
- * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.           *
- *****************************************************************************/
+ * You should have received a copy of the GNU General Public License         *
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
+ *****************************************************************************
+ */
 #ifndef PYTHON_PARSESESSION_H
 #define PYTHON_PARSESESSION_H
 #include <QtCore/QString>
 #include "parserexport.h"
 #include <language/duchain/indexedstring.h>
 #include <language/duchain/duchainpointer.h>
+#include <language/duchain/problem.h>
 #include <language/editor/simplecursor.h>
 #include <language/editor/documentrange.h>
 #include "ast.h"
 #include "kurl.h"
-#include <kdev-pg-memory-pool.h>
+#include "astdefaultvisitor.h"
 
-#include <language/interfaces/iproblem.h>
 #include <language/interfaces/iastcontainer.h>
 #include <language/editor/rangeinrevision.h>
 #include <language/editor/modificationrevision.h>
@@ -44,8 +39,8 @@ typedef QPair<KDevelop::DUContextPointer, KDevelop::RangeInRevision> SimpleUse;
 
 namespace Python
 {
-    class CodeAst;
-    class Ast;
+class CodeAst;
+class Ast;
 
 class KDEVPYTHONPARSER_EXPORT ParseSession : public IAstContainer
 {
@@ -56,12 +51,10 @@ public:
     void setContents( const QString& contents );
     QString contents() const;
     
-    KDevPG::MemoryPool* m_pool;
-    
     void setCurrentDocument(const IndexedString& url);
     IndexedString currentDocument();
 
-    QPair<CodeAst*, bool> parse( Python::CodeAst* ast );
+    QPair<CodeAst::Ptr, bool> parse();
     
     QList<KDevelop::ProblemPointer> m_problems;
     
@@ -74,7 +67,7 @@ public:
         Q_UNUSED(use);
     }
     
-    CodeAst* ast;
+    CodeAst::Ptr ast;
     
 private:
     QString m_contents;
