@@ -3,25 +3,20 @@
  * Copyright (c) 2007 Piyush verma <piyush.verma@gmail.com>                  *
  * Copyright (c) 2010-2012 Sven Brauch <svenbrauch@googlemail.com>           *
  *                                                                           *
- * Permission is hereby granted, free of charge, to any person obtaining     *
- * a copy of this software and associated documentation files (the           *
- * "Software"), to deal in the Software without restriction, including       *
- * without limitation the rights to use, copy, modify, merge, publish,       *
- * distribute, sublicense, and/or sell copies of the Software, and to        *
- * permit persons to whom the Software is furnished to do so, subject to     *
- * the following conditions:                                                 *
+ * This program is free software; you can redistribute it and/or             *
+ * modify it under the terms of the GNU General Public License as            *
+ * published by the Free Software Foundation; either version 2 of            *
+ * the License, or (at your option) any later version.                       *
  *                                                                           *
- * The above copyright notice and this permission notice shall be            *
- * included in all copies or substantial portions of the Software.           *
+ * This program is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
+ * GNU General Public License for more details.                              *
  *                                                                           *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,           *
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF        *
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                     *
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE    *
- * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION    *
- * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION     *
- * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.           *
- *****************************************************************************/
+ * You should have received a copy of the GNU General Public License         *
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
+ *****************************************************************************
+ */
 #ifndef PYTHON_PARSEJOB_H
 #define PYTHON_PARSEJOB_H
 
@@ -31,7 +26,7 @@
 
 #include <ksharedptr.h>
 #include <ktexteditor/range.h>
-#include <kdev-pg-memory-pool.h>
+#include <astdefaultvisitor.h>
 
 #include <language/backgroundparser/parsejob.h>
 #include <language/duchain/duchainpointer.h>
@@ -45,7 +40,6 @@ namespace Python
 {
 
 class LanguageSupport;
-
 class ParseSession;
 
 class ParseJob : public KDevelop::ParseJob
@@ -60,7 +54,7 @@ public:
     ParseJob(const IndexedString& url, ILanguageSupport* languageSupport );
     virtual ~ParseJob();
 
-    virtual CodeAst *ast() const;
+    virtual CodeAst* ast() const;
     bool wasReadFromDisk() const;
     static void eventuallyDoPEP8Checking(const IndexedString document, TopDUContext* topContext);
 
@@ -68,7 +62,7 @@ protected:
     virtual void run();
 
 private:
-    CodeAst *m_ast;
+    CodeAst::Ptr m_ast;
     bool m_readFromDisk;
     KDevelop::ReferencedTopDUContext m_duContext;
     KTextEditor::Range m_textRangeToParse;
